@@ -18,12 +18,11 @@ import warnings
 
 import xmlschema
 from xmlschema import XMLSchemaValidationError, ParkerConverter, \
-    BadgerFishConverter, AbderaConverter, JsonMLConverter
+    BadgerFishConverter, AbderaConverter, JsonMLConverter, UnorderedConverter
 
 from xmlschema.compat import unicode_type, ordered_dict_class
 from xmlschema.etree import etree_tostring, ElementTree, \
     etree_elements_assert_equal, lxml_etree, lxml_etree_element
-from xmlschema.qnames import XSI_TYPE
 from xmlschema.resources import fetch_namespaces
 
 from xmlschema.tests import XsdValidatorTestCase
@@ -92,6 +91,12 @@ def make_validator_test_class(test_file, test_args, test_num, schema_class, narr
 
         def check_etree_encode(self, root, converter=None, **kwargs):
             namespaces = kwargs.get('namespaces', {})
+
+            lossy = converter in (ParkerConverter, AbderaConverter)
+            losslessly = converter is JsonMLConverter
+            unordered = converter not in (AbderaConverter, JsonMLConverter) or \
+                kwargs.get('unordered', False)
+
             data1 = self.schema.decode(root, converter=converter, **kwargs)
             if isinstance(data1, tuple):
                 data1 = data1[0]  # When validation='lax'
@@ -117,20 +122,21 @@ def make_validator_test_class(test_file, test_args, test_num, schema_class, narr
 
             # Main check: compare original a re-encoded tree
             try:
-                etree_elements_assert_equal(root, elem1, strict=False)
+                etree_elements_assert_equal(root, elem1, strict=False, unordered=unordered)
             except AssertionError as err:
                 # If the check fails retry only if the converter is lossy (eg. ParkerConverter)
                 # or if the XML case has defaults taken from the schema or some part of data
                 # decoding is skipped by schema wildcards (set the specific argument in testfiles).
-                if converter not in (ParkerConverter, AbderaConverter, JsonMLConverter) and not skip_strict:
+                if skip_strict:
+                    pass  # can't ensure encode equivalence if the test case use defaults
+                elif lossy:
+                    pass  # can't check encode equivalence if the converter is lossy
+                elif losslessly:
                     if debug_mode:
                         pdb.set_trace()
-                    breakpoint()
                     raise AssertionError(str(err) + msg_tmpl % "encoded tree differs from original")
-                elif converter is ParkerConverter and any(XSI_TYPE in e.attrib for e in root.iter()):
-                    return  # can't check encode equivalence if xsi:type is provided
                 else:
-                    # Lossy or augmenting cases are checked after another decoding/encoding pass
+                    # Lossy or augmenting cases are checked with another decoding/encoding pass
                     data2 = self.schema.decode(elem1, converter=converter, **kwargs)
                     if isinstance(data2, tuple):
                         data2 = data2[0]
@@ -149,13 +155,17 @@ def make_validator_test_class(test_file, test_args, test_num, schema_class, narr
                         elem2 = elem2[0]
 
                     try:
-                        etree_elements_assert_equal(elem1, elem2, strict=False)
+                        etree_elements_assert_equal(elem1, elem2, strict=False, unordered=unordered)
                     except AssertionError as err:
                         if debug_mode:
                             pdb.set_trace()
                         raise AssertionError(str(err) + msg_tmpl % "encoded tree differs after second pass")
 
         def check_json_serialization(self, root, converter=None, **kwargs):
+            lossy = converter in (ParkerConverter, AbderaConverter)
+            unordered = converter not in (AbderaConverter, JsonMLConverter) or \
+                kwargs.get('unordered', False)
+
             data1 = xmlschema.to_json(root, schema=self.schema, converter=converter, **kwargs)
             if isinstance(data1, tuple):
                 data1 = data1[0]
@@ -168,16 +178,26 @@ def make_validator_test_class(test_file, test_args, test_num, schema_class, narr
             if isinstance(data2, tuple):
                 data2 = data2[0]
 
-            if converter is ParkerConverter and any(XSI_TYPE in e.attrib for e in root.iter()):
-                return  # can't check encode equivalence if xsi:type is provided
-            elif sys.version_info >= (3, 6):
+            if data2 != data1 and (skip_strict or lossy or unordered):
+                # Can't ensure decode equivalence if the test case use defaults,
+                # or the converter is lossy or the decoding is unordered.
+                return
+
+            if sys.version_info >= (3, 6):
+                if data1 != data2:
+                    print(data1)
+                    print(data2)
+                    print(converter, unordered)
                 self.assertEqual(data2, data1, msg_tmpl % "serialized data changed at second pass")
             else:
                 elem2 = xmlschema.from_json(data2, schema=self.schema, path=root.tag, converter=converter, **kwargs)
                 if isinstance(elem2, tuple):
                     elem2 = elem2[0]
+
                 try:
-                    self.assertIsNone(etree_elements_assert_equal(elem1, elem2, strict=False, skip_comments=True))
+                    self.assertIsNone(etree_elements_assert_equal(
+                        elem1, elem2, strict=False, skip_comments=True, unordered=unordered
+                    ))
                 except AssertionError as err:
                     self.assertIsNone(err, None)
 
@@ -241,21 +261,23 @@ def make_validator_test_class(test_file, test_args, test_num, schema_class, narr
             options = {'namespaces': namespaces, 'dict_class': ordered_dict_class}
 
             self.check_etree_encode(root, cdata_prefix='#', **options)  # Default converter
-            if narrow:
+            if not narrow:
                 self.check_etree_encode(root, ParkerConverter, validation='lax', **options)
                 self.check_etree_encode(root, ParkerConverter, validation='skip', **options)
                 self.check_etree_encode(root, BadgerFishConverter, **options)
                 self.check_etree_encode(root, AbderaConverter, **options)
                 self.check_etree_encode(root, JsonMLConverter, **options)
+                self.check_etree_encode(root, UnorderedConverter, cdata_prefix='#', **options)
 
             options.pop('dict_class')
             self.check_json_serialization(root, cdata_prefix='#', **options)
-            if narrow:
+            if not narrow:
                 self.check_json_serialization(root, ParkerConverter, validation='lax', **options)
                 self.check_json_serialization(root, ParkerConverter, validation='skip', **options)
                 self.check_json_serialization(root, BadgerFishConverter, **options)
                 self.check_json_serialization(root, AbderaConverter, **options)
                 self.check_json_serialization(root, JsonMLConverter, **options)
+                self.check_json_serialization(root, UnorderedConverter, **options)
 
         def check_decoding_and_encoding_with_lxml(self):
             xml_tree = lxml_etree.parse(xml_file)
@@ -284,21 +306,23 @@ def make_validator_test_class(test_file, test_args, test_num, schema_class, narr
                     'dict_class': ordered_dict_class,
                 }
                 self.check_etree_encode(root, cdata_prefix='#', **options)  # Default converter
-                if narrow:
+                if not narrow:
                     self.check_etree_encode(root, ParkerConverter, validation='lax', **options)
                     self.check_etree_encode(root, ParkerConverter, validation='skip', **options)
                     self.check_etree_encode(root, BadgerFishConverter, **options)
                     self.check_etree_encode(root, AbderaConverter, **options)
                     self.check_etree_encode(root, JsonMLConverter, **options)
+                    self.check_etree_encode(root, UnorderedConverter, cdata_prefix='#', **options)
 
                 options.pop('dict_class')
                 self.check_json_serialization(root, cdata_prefix='#', **options)
-                if narrow:
+                if not narrow:
                     self.check_json_serialization(root, ParkerConverter, validation='lax', **options)
                     self.check_json_serialization(root, ParkerConverter, validation='skip', **options)
                     self.check_json_serialization(root, BadgerFishConverter, **options)
                     self.check_json_serialization(root, AbderaConverter, **options)
                     self.check_json_serialization(root, JsonMLConverter, **options)
+                    self.check_json_serialization(root, UnorderedConverter, **options)
 
         def check_validate_and_is_valid_api(self):
             if expected_errors:
